import psycopg2
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, GObject
import datetime

class MyWindow(Gtk.Window):
    def __init__(self):
        Gtk.Window.__init__(self, title = "Time Keeper")
        self.set_border_width(3)
        self.set_default_size(400, 300)

        self.timekeeper_tid = None
        self.durationkeeper_tid = None
        self.start_time = datetime.datetime
        self.timediff = datetime.timedelta

        vbox = Gtk.Box(orientation = Gtk.Orientation.VERTICAL, spacing = 6)
        self.add(vbox)

        hbox = Gtk.Box(spacing = 50)
        vbox.add(hbox)
        self.descLabelStrt = Gtk.Label("Current Time : ", xalign = 0)
        self.timeStarted = Gtk.Label()
        hbox.pack_start(self.descLabelStrt, True, True, 0)
        hbox.pack_start(self.timeStarted, False, True, 0)

        hbox = Gtk.Box(spacing = 50)
        vbox.add(hbox)
        self.timeStopped = Gtk.Label("Nothing to display yet")
        self.descLabelStop = Gtk.Label("Last session ended at : ", xalign = 0)
        hbox.pack_start(self.descLabelStop, True, True, 0)
        hbox.pack_start(self.timeStopped, False, True, 0)

        hbox = Gtk.Box(spacing = 50)
        vbox.add(hbox)
        self.elapseTimeLabel = Gtk.Label("Time Elapsed : ", xalign = 0)
        self.elapseDuration = Gtk.Label("Nothing to display yet")
        hbox.pack_start(self.elapseTimeLabel, True, True, 0)
        hbox.pack_start(self.elapseDuration, False, True, 0)

        strtButton = Gtk.Button(label="Start", valign = Gtk.Align.CENTER)
        strtButton.connect("clicked", self.startRecording)

        stopButton = Gtk.Button(label="Stop", valign = Gtk.Align.CENTER)
        stopButton.connect("clicked", self.stopRecording)

        vbox.pack_start(strtButton, False, True, 0)
        vbox.pack_start(stopButton, False, True, 0)

        self.desc = Gtk.TextView()

        vbox.pack_start(self.desc, True, True, 0)
        self.textbuffer = self.desc.get_buffer()
        self.textbuffer.set_text("Enter here how you utilised your time.")

        self.timekeeper_tid = GObject.timeout_add(100, self.keepTime, None)

    def startRecording(self, button):
        GObject.source_remove(self.timekeeper_tid)
        self.descLabelStrt.set_text("Recording time started at : ")
        self.durationkeeper_tid = GObject.timeout_add(100, self.keepDuration, None)
        self.start_time = datetime.datetime.now()

    def stopRecording(self, button):
        self.descLabelStrt.set_text("Current time : ")
        self.timekeeper_tid = GObject.timeout_add(100, self.keepTime, None)
        self.timeStopped.set_text(str(datetime.datetime.now().hour) + " : " + str(datetime.datetime.now().minute) + " : " + str(datetime.datetime.now().second))
        GObject.source_remove(self.durationkeeper_tid)
        desc_text = self.textbuffer.get_text(self.textbuffer.get_start_iter(), self.textbuffer.get_end_iter(), True)

        conn = psycopg2.connect("dbname=aitestdb user=postgres password=superman123")
        cur = conn.cursor()
        print(cur.mogrify("""SELECT %s, %s, %s, %s""", (self.start_time.date(), self.start_time.time(), datetime.datetime.now() - self.start_time, desc_text)))
        cur.execute("""INSERT INTO sessionlogs(sessiondate, sessiontime, sessionduration, sessiondesc)
                    VALUES(%s, %s, %s, %s);""", (self.start_time.date(), self.start_time.time(), datetime.datetime.now() - self.start_time, desc_text))
        conn.commit()
        cur.close()
        conn.close()

        self.textbuffer.set_text("Enter here how you utilised your time.")

    def keepDuration(self, whatevs):
        self.timediff = datetime.datetime.now() - self.start_time
        seconds = self.timediff.seconds % 60
        minutes = int((self.timediff.seconds / 60) % 60)
        hours = int(minutes / 60)
                
        self.elapseDuration.set_text(str(hours) + " Hours " + str(minutes) + " Minutes " + str(seconds) + " Seconds ")
        return True        

    def keepTime(self, whatevs):
        self.timeStarted.set_text(str(datetime.datetime.now().hour) + " : " + str(datetime.datetime.now().minute) + " : " + str(datetime.datetime.now().second))
        return True

win = MyWindow()
win.connect("delete-event", Gtk.main_quit)
win.show_all()
Gtk.main()
